/*
WYSIWYG-BBCODE editor
Copyright (c) 2009, Jitbit Sotware, http://www.jitbit.com/
PROJECT HOME: http://wysiwygbbcode.codeplex.com/
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
	* Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.
	* Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.
	* Neither the name of the <organization> nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY Jitbit Software ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL Jitbit Software BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

var wswgEditor = new function () {

	this.getEditorDoc = function () { return myeditor; }
	this.getIframe = function () { return ifm; }
	this.IsEditorVisible = function () { return editorVisible; }

	var myeditor, ifm;
	var body_id, textboxelement;
	var content;
	var isIE = /msie|MSIE/.test(navigator.userAgent); //detects OLD ie versions, not IE11, but thats ok, ie11 is mostly a fine browser....
	var isChrome = /Chrome/.test(navigator.userAgent);
	var isSafari = /Safari/.test(navigator.userAgent) && !isChrome;
	var isFirefox = navigator.userAgent.toLowerCase().indexOf('firefox') > -1;
	var browser = isIE || window.opera;
	var textRange;
	var editorVisible = false;
	var enableWysiwyg = false;

	var scriptElements = document.getElementsByTagName('script');
	var curScriptUrl = scriptElements[scriptElements.length - 1].src;

	function rep(re, str) {
		content = content.replace(re, str);
	}

	this.initEditor = function (textarea_id, wysiwyg) {
		if (wysiwyg != undefined)
			enableWysiwyg = wysiwyg;
		else
			enableWysiwyg = true;
		body_id = textarea_id;
		textboxelement = document.getElementById(body_id);
		textboxelement.className += " editorBBCODE";
		if (enableWysiwyg) {
			if (!document.getElementById("rte")) { //to prevent recreation
				ifm = document.createElement("iframe");
				ifm.setAttribute("id", "rte");
				ifm.setAttribute("frameBorder", "0");
				if (isIE) {
					// Ugly IE hacks due to IE not inheriting document.domain from parent
					// checks if document.domain is set by comparing the host name against document.domain
					source = "javascript:document.write(\"<head><script>document.domain=\\\"" + document.domain + "\\\";</script></head><body></body>\")";
					ifm.setAttribute("src", source);
				}
				ifm.style.width = textboxelement.style.width;
				ifm.style.height = textboxelement.style.height;
				textboxelement.parentNode.insertBefore(ifm, textboxelement);
				textboxelement.style.display = 'none';
			}
			if (ifm) {
				InitIframe();
			} else
				setTimeout('InitIframe()', 100);
		}
	}

	function InitIframe() {
		myeditor = ifm.contentWindow.document;
		myeditor.designMode = "on";
		myeditor.open();
		myeditor.write('<html><head><link href="https://fonts.googleapis.com/css?family=Open+Sans:400,600&subset=latin,latin-ext" rel="stylesheet" type="text/css"><style>body{font:14px "Open Sans", sans-serif;background-color:#fff;margin:0}p{margin:0;}a,a:visited{color:#1566ad}</style></head>');
		myeditor.write('<body>');
		myeditor.write('</body></html>');
		myeditor.close();
		myeditor.body.contentEditable = true;

		//ifm.contentEditable = true;
		if (myeditor.attachEvent) {
			myeditor.attachEvent("onkeypress", kp);
		}
		else if (myeditor.addEventListener) {
			myeditor.addEventListener("keypress", kp, true);
			myeditor.body.addEventListener('paste', handlePaste);
		}
		wswgEditor.ShowEditor();
	}

	function handlePaste(evt) {
		if (!evt.clipboardData) return;
		if (isFirefox) return;//firefox works out of the box (latest versions)
		for (var i = 0; i < evt.clipboardData.items.length; i++) {
			if (evt.clipboardData.items[i].kind == "file" && evt.clipboardData.items[i].type == "image/png") {
				var imageFile = evt.clipboardData.items[i].getAsFile();
				var fileReader = new FileReader();
				fileReader.onloadend = function () {
					AddImageByUrl(fileReader.result);
				}
				fileReader.readAsDataURL(imageFile);
				break;
			}
		}
		return false;
	}

	function RemovePlaceholder() {
		var pElement = myeditor.getElementById("placeholderspan");
		if (pElement) pElement.parentNode.removeChild(pElement);
		if(myeditor.addEventListener)
			ifm.contentWindow.removeEventListener('focus', RemovePlaceholder);
	}

	this.ShowEditor = function () {
		if (!enableWysiwyg) return;
		editorVisible = true;
		ifm.style.display = '';
		textboxelement.style.display = 'none';

		content = document.getElementById(body_id).value;
		bbcode2html();
		var placeholderText = document.getElementById(body_id).placeholder;
		if(content || !placeholderText)
			myeditor.body.innerHTML = content;
		else { //write placeholder (if no content and placeholder is not empty)
			if (myeditor.addEventListener) { //screw old browsers, no placeholder for them
				myeditor.body.innerHTML = '<span style="color:#aaa" id="placeholderspan">' + placeholderText + '</span>';
				ifm.contentWindow.addEventListener('focus', RemovePlaceholder); //remove placeholder on focus
			}
		}
	}

	this.SwitchEditor = function () {
		if (editorVisible) {
			this.doCheck();
			ifm.style.display = 'none';
			textboxelement.style.display = '';
			editorVisible = false;
			textboxelement.focus();
		}
		else {
			if (enableWysiwyg && ifm) {
				this.ShowEditor();
				ifm.contentWindow.focus();
			}
		}
	}

	function html2bbcode() {
		//escape brackets
		rep(/\[/g, "\\[");
		rep(/\]/g, "\\]");

		//first, parse SOME data using browser's HTML engine rather than regex
		var imgs = myeditor.body.getElementsByTagName('img');
		while (imgs.length > 0) {
			imgs[0].parentNode.replaceChild(document.createTextNode("[img"
				+ (imgs[0].hasAttribute("width") ? " width=" + imgs[0].getAttribute("width") : "")
				+ (imgs[0].hasAttribute("height") ? " height=" + imgs[0].getAttribute("height") : "")
				+ "]" + imgs[0].getAttribute("src") + "[/img]"), imgs[0]);
		}
		var anchors = myeditor.body.getElementsByTagName('a');
		while (anchors.length > 0) {
			var span = document.createElement('span');
			span.innerHTML = "[url=" + anchors[0].href + "]" + anchors[0].innerHTML + "[/url]";
			anchors[0].parentNode.replaceChild(span, anchors[0]);
		}

		content = myeditor.body.innerHTML;

		//now - regexs
		rep(/<\/(strong|b)>/gi, "[/b]");
		rep(/<(strong|b)(\s[^<>]*)?>/gi, "[b]");
		rep(/<\/(em|i)>/gi, "[/i]");
		rep(/<(em|i)(\s[^<>]*)?>/gi, "[i]");
		rep(/<\/u>/gi, "[/u]");
		rep(/\n/gi, " ");
		rep(/\r/gi, " ");
		rep(/<u(\s[^<>]*)?>/gi, "[u]");
		rep(/<div><br(\s[^<>]*)?>/gi, "<div>"); //chrome-safari fix to prevent double linefeeds
		rep(/<p><br>/gi, "<p>"); //ie fix to prevent double linefeeds
		rep(/<br(\s[^<>]*)?>/gi, "\n");
		rep(/<p(\s[^<>]*)?>/gi, "");
		rep(/<\/p>/gi, "\n");
		rep(/<ul([^>]*)>/gi, "[ul]");
		rep(/<\/ul>/gi, "[/ul]");
		rep(/<ol>/gi, "[ol]");
		rep(/<\/ol>/gi, "[/ol]");
		rep(/<li([^>]*)>/gi, "[li]");
		rep(/<\/li>/gi, "[/li]");
		rep(/<\/div>\s*<div([^<>]*)>/gi, "</span>\n<span$1>"); //chrome-safari fix to prevent double linefeeds
		rep(/<div([^<>]*)>/gi, "\n<span$1>");
		rep(/<\/div>/gi, "</span>\n");
		rep(/<hr[^>]*>/gi, "[hr]");

		rep(/<table([^<>]*)>/gi, "[table]");
		rep(/<\/table>/gi, "[/table]");
		rep(/<tr([^<>]*)>/gi, "[tr]");
		rep(/<td([^<>]*)>/gi, "[td]");
		rep(/<pre>([^<>]*?)<\/pre>/gi, "[code]$1[/code]");
		rep(/<code>([^<>]*?)<\/code>/gi, "[code]$1[/code]");
		rep(/<blockquote>/gi, "[quote]");
		rep(/<\/blockquote>/gi, "[/quote]");

		rep(/&nbsp;/gi, " ");
		rep(/&quot;/gi, "\"");
		rep(/&amp;/gi, "&");

		//remove style & script & xml tags (xml can be added when pasting from Word/Outlook
		rep(/<script.*?>[\s\S]*?<\/script>/gi, "");
		rep(/<style.*?>[\s\S]*?<\/style>/gi, "");
		rep(/<xml>[\s\S]*?<\/xml>/gi, "");

		//remove [if] blocks (when pasted from outlook etc)
		rep(/<!--\[if[\s\S]*?<!\[endif\]-->/gi, "");

		var sc, sc2;
		do {
			sc = content;
			rep(/<font\s[^<>]*?color=\"?([^<>]*?)\"?(\s[^<>]*)?>([^<>]*?)<\/font>/gi, "[color=$1]$3[/color]");
			if (sc == content)
				rep(/<font[^<>]*>([^<>]*?)<\/font>/gi, "$1");
			sc2 = content;
			rep(/<span\s[^<>]*?style=\"?font-weight: ?bold;?\"?\s*([^<]*?)<\/span>/gi, "[b]<span style=$1</span>[/b]");
			rep(/<span\s[^<>]*?style=\"?font-size: ?([^<>]*?);\"?\s*([^<]*?)<\/span>/gi, "[size=$1]<span style=$2</span>[/size]");
			rep(/<span\s[^<>]*?style=\"?font-style: ?italic;?\"?\s*([^<]*?)<\/span>/gi, "[i]<span style=$1</span>[/i]");
			rep(/<span\s[^<>]*?style=\"?font-style: ?normal;?\"?\s*([^<]*?)<\/span>/gi, "<span style=$1</span>");
			rep(/<span\s[^<>]*?style=\"?text-decoration: ?underline;?\"?\s*([^<]*?)<\/span>/gi, "[u]<span style=$1</span>[/u]");
			rep(/<span\s[^<>]*?style=\"?text-decoration: ?none;?\"?\s*([^<]*?)<\/span>/gi, "<span style=$1</span>");
			rep(/<span\s[^<>]*?style=\"?color: ?([^<>]*?);\"?\s*([^<]*?)<\/span>/gi, "[color=$1]<span style=$2</span>[/color]");
			rep(/<span\s[^<>]*?style=\"?font-family: ?([^<>]*?);\"?\s*([^<]*?)<\/span>/gi, "[font=$1]<span style=$2</span>[/font]");
			rep(/<span\s[^<>]*?style=\"?\"?>([^<>]*?)<\/span>/gi, "$1");
			if (sc2 == content) {
				rep(/<span[^<>]*>([^<>]*?)<\/span>/gi, "$1");
				sc2 = content;
			}
		} while (sc != content)
		rep(/<[^<>]*>/gi, "");
		rep(/&lt;/gi, "<");
		rep(/&gt;/gi, ">");

		//clean up empty tags
		do {
			sc = content;
			rep(/\[b\]\[\/b\]/gi, "");
			rep(/\[i\]\[\/i\]/gi, "");
			rep(/\[u\]\[\/u\]/gi, "");
			rep(/\[quote[^\]]*\]\[\/quote\]/gi, "");
			rep(/\[code\]\[\/code\]/gi, "");
			rep(/\[url=([^\]]+)\]\[\/url\]/gi, "");
			rep(/\[img\]\[\/img\]/gi, "");
			rep(/\[color=([^\]]*)\]\[\/color\]/gi, "");
		} while (sc != content)
	}

	this.BBCodeToHtml = function (str) {
		content = str;
		bbcode2html();
		return content;
	}

	function bbcode2html() {
		// example: [b] to <strong>
		rep(/\</gi, "&lt;"); //removing html tags
		rep(/\>/gi, "&gt;");

		rep(/\n/gi, "<br />");
		rep(/\[ul\]/gi, "<ul>");
		rep(/\[\/ul\]/gi, "</ul>");
		rep(/\[list\]/gi, "<ul>");
		rep(/\[\/list\]/gi, "</ul>");
		rep(/\[ol\]/gi, "<ol>");
		rep(/\[\/ol\]/gi, "</ol>");
		rep(/\[li\]/gi, "<li>");
		rep(/\[\/li\]/gi, "</li>");
		rep(/\[hr\]/gi, "<hr>");

		rep(/\[table\]/gi, "<table border='1'>");
		rep(/\[\/table\]/gi, "</table>");
		rep(/\[tr\]/gi, "<tr>");
		rep(/\[\/tr\]/gi, "</tr>");
		rep(/\[td\]/gi, "<td>");
		rep(/\[\/td\]/gi, "</td>");

		rep(/\[quote\]/gi, "<blockquote>");
		rep(/\[\/quote\]/gi, "</blockquote>");

		if (browser) {
			rep(/\[b\]/gi, "<strong>");
			rep(/\[\/b\]/gi, "</strong>");
			rep(/\[i\]/gi, "<em>");
			rep(/\[\/i\]/gi, "</em>");
			rep(/\[u\]/gi, "<u>");
			rep(/\[\/u\]/gi, "</u>");
		} else {
			rep(/\[b\]/gi, "<span style=\"font-weight: bold;\">");
			rep(/\[i\]/gi, "<span style=\"font-style: italic;\">");
			rep(/\[u\]/gi, "<span style=\"text-decoration: underline;\">");
			rep(/\[\/(b|i|u)\]/gi, "</span>");
		}
		rep(/\[img([^\]]*)\]/gi, "<img$1 src=\"");
		rep(/\[\/img\]/gi, "\" border=\"0\">");
		var sc;
		do {
			sc = content;
			rep(/\[url=([^\]]+)\]([\s\S]*?)\[\/url\]/gi, "<a href=\"$1\">$2</a>");
			rep(/\[url\]([\s\S]*?)\[\/url\]/gi, "<a href=\"$1\">$1</a>");
			if (browser) {
				rep(/\[color=([^\]]*?)\]([\s\S]*?)\[\/color\]/gi, "<font color=\"$1\">$2</font>");
				rep(/\[font=([^\]]*?)\]([\s\S]*?)\[\/font\]/gi, "<font face=\"$1\">$2</font>");
			} else {
				rep(/\[color=([^\]]*?)\]([\s\S]*?)\[\/color\]/gi, "<span style=\"color: $1;\">$2</span>");
				rep(/\[font=([^\]]*?)\]([\s\S]*?)\[\/font\]/gi, "<span style=\"font-family: $1;\">$2</span>");
			}
			rep(/\[size=([^\]]*?)\]([\s\S]*?)\[\/size\]/gi, "<span style=\"font-size: $1;\">$2</span>");
			rep(/\[code\]([\s\S]*?)\[\/code\]/gi, "<pre>$1</pre>&nbsp;");
			rep(/\[pre\]([\s\S]*?)\[\/pre\]/gi, "<pre>$1</pre>&nbsp;");
		} while (sc != content);
		rep(/\\\[/g, "[");
		rep(/\\\]/g, "]");
	}

	this.doCheck = function () {
		if (!enableWysiwyg) return;
		RemovePlaceholder();
		if (editorVisible) {
			content = myeditor.body.innerHTML;
			html2bbcode();
			document.getElementById(body_id).value = content;
		}
	}

	function stopEvent(evt) {
		evt || window.event;
		if (evt.stopPropagation) {
			evt.stopPropagation();
			evt.preventDefault();
		} else if (typeof evt.cancelBubble != "undefined") {
			evt.cancelBubble = true;
			evt.returnValue = false;
		}
		return false;
	}

	function kp(e) {
		if (e.keyCode == 13) {
			if (isIE) {
				var r = myeditor.selection.createRange();
				if (r.parentElement().tagName.toLowerCase() != "li") {
					r.pasteHTML('<br/>');
					if (r.move('character'))
						r.move('character', -1);
					r.select();
					stopEvent(e);
					return false;
				}
			}
		}
	}
	this.InsertYoutube = function () {
		this.InsertText(" http://www.youtube.com/watch?v=XXXXXXXXXXX ");
	}
	this.InsertText = function (txt) {
		if (editorVisible)
			insertHtml(txt);
		else
			AddTag('', txt);
	}

	this.doClick = function (command, arg) {
		if (editorVisible) {
			ifm.contentWindow.focus();
			myeditor.execCommand(command, false, arg);
			ifm.contentWindow.focus();
			ifm.contentWindow.document.body.focus();
		}
		else {
			switch (command) {
				case 'bold':
					AddTag('[b]', '[/b]'); break;
				case 'italic':
					AddTag('[i]', '[/i]'); break;
				case 'underline':
					AddTag('[u]', '[/u]'); break;
				case 'InsertUnorderedList':
					AddTag('[ul][li]', '[/li][/ul]'); break;
				case 'InsertOrderedList':
					AddTag('[ol][li]', '[/li][/ol]'); break;
			}
		}
	}

	this.doCode = function () {
		if (editorVisible) {
			if (window.getSelection) {
				var selection = ifm.contentWindow.getSelection().getRangeAt(0);
				var selectedText = selection.extractContents();
				var span = ifm.contentWindow.document.createElement("pre");
				span.appendChild(selectedText);
				selection.insertNode(span);
			}
		}
		else {
			AddTag('[code]', '[/code]');
		}
	}

	function doColor(color) {
		ifm.contentWindow.focus();
		if (isIE) {
			textRange = ifm.contentWindow.document.selection.createRange();
			textRange.select();
		}
		if (color.indexOf('#') != 0) color = '#' + color; //firefox workaround
		myeditor.execCommand('forecolor', false, color);
		ifm.contentWindow.focus();
		ifm.contentWindow.document.body.focus();
	}

	this.doLink = function () {
		ifm.contentWindow.focus();
		var mylink = prompt("Enter a URL:", "http://");
		if (editorVisible) {
			if ((mylink != null) && (mylink != "")) {
				if (isIE) { //IE
					var range = ifm.contentWindow.document.selection.createRange();
					if (range.text == '') {
						range.pasteHTML("<a href='" + mylink + "'>" + mylink + "</a>");
					}
					else
						myeditor.execCommand("CreateLink", false, mylink);
				}
				else if (window.getSelection) { //FF
					var userSelection = ifm.contentWindow.getSelection().getRangeAt(0);
					if (userSelection.toString().length == 0)
						insertHtml("<a href='" + mylink + "'>" + mylink + "</a>");
					else
						myeditor.execCommand("CreateLink", false, mylink);
				}
				else
					myeditor.execCommand("CreateLink", false, mylink);
			}
		}
		else {
			AddTag('[url=' + mylink + ']', '[/url]');
		}
	}
	this.doImage = function (postbackUrl) {
		var strHtml = '<!DOCTYPE html><html><head> \
		<link href="' + curScriptUrl.replace("editor.js", "main.min.css") + '" type="text/css" rel="stylesheet" /> \
		</head> \
		<body style="min-width:0"><form method="post" action="' + postbackUrl + '" enctype = "multipart/form-data"> \
		<table width="100%"> \
		<tr><td><input type="radio" id="fromWeb" name="upload" value="0" checked="checked" onclick="document.getElementById(\'lblFileUpload\').style.visibility=\'hidden\';document.getElementById(\'txUrl\').style.visibility=\'visible\';" /> <label for="fromWeb">From the Web</label></td> \
			<td><input type="text" id="txUrl" name="txUrl" placeholder="http://"/> \
		</td></tr> \
		<tr><td><input type="radio" id="uploadFromPC" name="upload" value="1" onclick="document.getElementById(\'txUrl\').style.visibility=\'hidden\';document.getElementById(\'lblFileUpload\').style.visibility=\'visible\';" /> <label for="uploadFromPC">From my computer</label></td> \
			<td><input type="file" id="fileUpload" name="fileUpload" id="fileUpload" class="fancyfile"/><label for="fileUpload" id="lblFileUpload" style="visibility:hidden;">Select a file...</label></td></tr>\
		<tr><td colspan="2" align="right" style="padding-bottom:0"><input type="submit" id="btnAddPic" value="Add picture"/></td></tr></table></form> \
		<script src="' + curScriptUrl.replace("editor.js", "fancyfile.js") + '"></script>\
		</body></html>';

		$.fancybox('<iframe id="frmImage" width="500" height="145" style="display:block"></iframe>',
		{
			'transitionIn': 'fade',
			'transitionOut': 'none',
			padding: 0
		});
		$.fancybox.init(); //reinit - safari fix
		var ifr = document.getElementById('frmImage');
		var ifrDoc = ifr.contentWindow.document;
		ifrDoc.open();
		ifrDoc.write(strHtml);
		ifrDoc.close();

		$("#btnAddPic", ifrDoc).click(function () {
			if ($('#fromWeb', ifrDoc).is(':checked')) { //from web
				AddImageByUrl($('#txUrl', ifrDoc).val());
				$.fancybox.close();
				return false; //to prevent form submit
			}
			else //upload from computer
			{
				ifr.onload = function () {
					AddImageByUrl($(ifr.contentWindow.document.body).html());
					$.fancybox.close();
				};
			}
		});
		return;
	}

	function AddImageByUrl(url) {
		if (editorVisible) {
			ifm.contentWindow.focus();
			if ((url != null) && (url != "")) {
				myeditor.execCommand('InsertImage', false, url);
			}
		}
		else {
			AddTag('[img]', url + '[/img]');
		}
	}

	function insertHtml(html) {
		if (window.getSelection) { // IE9 and non-IE
			var sel = ifm.contentWindow.getSelection();
			if (sel.getRangeAt && sel.rangeCount) {
				var range = sel.getRangeAt(0);
				sel.removeAllRanges();
				range.deleteContents();
				var el = ifm.contentWindow.document.createElement("div");
				el.innerHTML = html;
				var frag = ifm.contentWindow.document.createDocumentFragment(), node;
				while ((node = el.firstChild)) {
					frag.appendChild(node);
				}
				range.insertNode(frag);
				range.collapse(false);
				sel.addRange(range);
			}
		} else if (ifm.contentWindow.document.selection && ifm.contentWindow.document.selection.type != "Control") {
			// IE < 9
			ifm.contentWindow.focus();
			ifm.contentWindow.document.selection.createRange().pasteHTML(html);
		}
	}

	//textarea-mode functions
	function MozillaInsertText(element, text, pos) {
		element.value = element.value.slice(0, pos) + text + element.value.slice(pos);
	}

	function AddTag(t1, t2) {
		var element = textboxelement;
		if (isIE) {
			if (document.selection) {
				element.focus();

				var txt = element.value;
				var str = document.selection.createRange();

				if (str.text == "") {
					str.text = t1 + t2;
				}
				else if (txt.indexOf(str.text) >= 0) {
					str.text = t1 + str.text + t2;
				}
				else {
					element.value = txt + t1 + t2;
				}
				str.select();
			}
		}
		else if (typeof (element.selectionStart) != 'undefined') {
			var sel_start = element.selectionStart;
			var sel_end = element.selectionEnd;
			MozillaInsertText(element, t1, sel_start);
			MozillaInsertText(element, t2, sel_end + t1.length);
			element.selectionStart = sel_start;
			element.selectionEnd = sel_end + t1.length + t2.length;
			element.focus();
		}
		else {
			element.value = element.value + t1 + t2;
		}
	}

	//=======color picker
	function getScrollY() { var scrOfX = 0, scrOfY = 0; if (typeof (window.pageYOffset) == 'number') { scrOfY = window.pageYOffset; scrOfX = window.pageXOffset; } else if (document.body && (document.body.scrollLeft || document.body.scrollTop)) { scrOfY = document.body.scrollTop; scrOfX = document.body.scrollLeft; } else if (document.documentElement && (document.documentElement.scrollLeft || document.documentElement.scrollTop)) { scrOfY = document.documentElement.scrollTop; scrOfX = document.documentElement.scrollLeft; } return scrOfY; }

	document.write("<style type='text/css'>.colorpicker201{visibility:hidden;display:none;position:absolute;background:#FFF;z-index:999;filter:progid:DXImageTransform.Microsoft.Shadow(color=#D0D0D0,direction=135);}.o5582brd{padding:0;width:12px;height:14px;border-bottom:solid 1px #DFDFDF;border-right:solid 1px #DFDFDF;}a.o5582n66,.o5582n66,.o5582n66a{font-family:arial,tahoma,sans-serif;text-decoration:underline;font-size:9px;color:#666;border:none;}.o5582n66,.o5582n66a{text-align:center;text-decoration:none;}a:hover.o5582n66{text-decoration:none;color:#FFA500;cursor:pointer;}.a01p3{padding:1px 4px 1px 2px;background:whitesmoke;border:solid 1px #DFDFDF;}</style>");

	function getTop2() { csBrHt = 0; if (typeof (window.innerWidth) == 'number') { csBrHt = window.innerHeight; } else if (document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight)) { csBrHt = document.documentElement.clientHeight; } else if (document.body && (document.body.clientWidth || document.body.clientHeight)) { csBrHt = document.body.clientHeight; } ctop = ((csBrHt / 2) - 115) + getScrollY(); return ctop; }
	var nocol1 = "&#78;&#79;&#32;&#67;&#79;&#76;&#79;&#82;",
clos1 = "X";

	function getLeft2() { var csBrWt = 0; if (typeof (window.innerWidth) == 'number') { csBrWt = window.innerWidth; } else if (document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight)) { csBrWt = document.documentElement.clientWidth; } else if (document.body && (document.body.clientWidth || document.body.clientHeight)) { csBrWt = document.body.clientWidth; } cleft = (csBrWt / 2) - 125; return cleft; }

	//function setCCbldID2(val, textBoxID) { document.getElementById(textBoxID).value = val; }
	function setCCbldID2(val) { if (editorVisible) doColor(val); else AddTag('[color=#' + val + ']', '[/color]'); }

	function setCCbldSty2(objID, prop, val) {
		switch (prop) {
			case "bc": if (objID != 'none') { document.getElementById(objID).style.backgroundColor = val; }; break;
			case "vs": document.getElementById(objID).style.visibility = val; break;
			case "ds": document.getElementById(objID).style.display = val; break;
			case "tp": document.getElementById(objID).style.top = val; break;
			case "lf": document.getElementById(objID).style.left = val; break;
		}
	}

	this.putOBJxColor2 = function (Samp, pigMent, textBoxId) { if (pigMent != 'x') { setCCbldID2(pigMent, textBoxId); setCCbldSty2(Samp, 'bc', pigMent); } setCCbldSty2('colorpicker201', 'vs', 'hidden'); setCCbldSty2('colorpicker201', 'ds', 'none'); if (isIE) { event.preventDefault ? event.preventDefault() : event.returnValue = false; } }

	this.showColorGrid2 = function (Sam, textBoxId) {
		var objX = new Array('00', '33', '66', '99', 'CC', 'FF');
		var c = 0;
		var xl = '"' + Sam + '","x", "' + textBoxId + '"'; var mid = '';
		mid += '<table class="editorColorPick"><tr>';
		mid += "<td colspan='9' align='left' style='margin:0;padding:2px;height:12px;' ><input class='o5582n66' type='text' size='12' id='o5582n66' value='#FFFFFF'><input class='o5582n66a' type='text' size='2' style='width:14px;' id='o5582n66a' onclick='alert(\"click on selected swatch below...\");' value='' style='border:solid 1px #666;'></td><td colspan='9' align='right'><a class='o5582n66' href='javascript:void(0)' onclick='wswgEditor.putOBJxColor2(" + xl + ")'><span class='a01p3'>" + clos1 + "</span></a></td></tr><tr>";
		var br = 1;
		for (o = 0; o < 6; o++) {
			mid += '</tr><tr>';
			for (y = 0; y < 6; y++) {
				if (y == 3) { mid += '</tr><tr>'; }
				for (x = 0; x < 6; x++) {
					var grid = '';
					grid = objX[o] + objX[y] + objX[x];
					var b = "'" + Sam + "','" + grid + "', '" + textBoxId + "'";
					mid += '<td class="o5582brd" style="background-color:#' + grid + '"><a class="o5582n66"  href="javascript:void(0)" onclick="wswgEditor.putOBJxColor2(' + b + ');" onmouseover=javascript:document.getElementById("o5582n66").value="#' + grid + '";javascript:document.getElementById("o5582n66a").style.backgroundColor="#' + grid + '";  title="#' + grid + '"><div style="width:12px;height:14px;"></div></a></td>';
					c++;
				}
			}
		}
		mid += "</tr></table>";
		//var ttop=getTop2();
		//setCCbldSty2('colorpicker201','tp',ttop);
		//document.getElementById('colorpicker201').style.left=getLeft2();
		document.getElementById('colorpicker201').innerHTML = mid;
		setCCbldSty2('colorpicker201', 'vs', 'visible');
		setCCbldSty2('colorpicker201', 'ds', 'inline');
	}

}