﻿/*	based on tags plugin by XOXCO Inc (c), Licensed under MIT (http://www.opensource.org/licenses/mit-license.php)
	MODIFIED BY JITBIT to support "name|id" tag format, where "id" - is the tag's id */

(function ($) {

	var delimiter = new Array();
	var tags_callbacks = new Array();
	var linkFormats = new Array();

	//"value" should be in the form "name|id"
	$.fn.addTag = function (value, options) {
		options = jQuery.extend({ focus: false, callback: true }, options);
		this.each(function () {
			var id = $(this).attr('id');

			var tagslist = $(this).val().split(delimiter[id]);
			if (tagslist[0] == '') {
				tagslist = new Array();
			}

			value = jQuery.trim(value);

			if (options.unique) {
				var skipTag = $(tagslist).tagExist(value);
			} else {
				var skipTag = false;
			}

			if (skipTag != true) {
				if (value != '') {
					//"name|id" format
					var lineCharPos = value.indexOf("|");
					var tagName = value.substr(0, lineCharPos);
					var	tagId = value.substr(lineCharPos + 1);

					var element = (linkFormats[id] == "") ?
						$('<span>').text(tagName) :
						$('<a>', { href: linkFormats[id] + tagId, text: tagName });

					$('<span>').addClass('tag').append(
							element,
							'&nbsp;&nbsp;',
							$('<a>', {
								href: 'javascript:void(0)',
								title: 'Remove this one',
								text: 'x'
							}).addClass('remove').click(function () {
								return $('#' + id).removeTag(escape(value));
							})
						).insertBefore('#' + id + '_addTag');

					tagslist.push(value);

					$('#' + id + '_tag').val('');
					if (options.focus) {
						$('#' + id + '_tag').focus();
					} else {
						$('#' + id + '_tag').blur();
					}

					if (options.callback && tags_callbacks[id] && tags_callbacks[id]['onAddTag']) {
						var f = tags_callbacks[id]['onAddTag'];
						f(tagId, tagName);
					}
					if (tags_callbacks[id] && tags_callbacks[id]['onChange']) {
						var i = tagslist.length;
						var f = tags_callbacks[id]['onChange'];
						f($(this), tagslist[i]);
					}
				}
			}
			else { //skip tag - lets clear the input field from typed text
				$('#' + id + '_tag').val('');
			}
			$.fn.tagsInput.updateTagsField(this, tagslist);

		});

		return false;
	};

	$.fn.removeTag = function (value) {
		value = unescape(value);

		//"name|id" format
		var lineCharPos = value.indexOf("|");
		var tagName = value.substr(0, lineCharPos);
		var tagId = value.substr(lineCharPos + 1);

		this.each(function () {
			var id = $(this).attr('id');

			var old = $(this).val().split(delimiter[id]);



			$('#' + id + '_tagsinput .tag').remove();
			str = '';
			for (i = 0; i < old.length; i++) {
				if (old[i] != value) {
					str = str + delimiter[id] + old[i];
				}
			}

			$.fn.tagsInput.importTags(this, str);

			if (tags_callbacks[id] && tags_callbacks[id]['onRemoveTag']) {
				var f = tags_callbacks[id]['onRemoveTag'];
				f(tagId);
			}
		});

		return false;
	};

	$.fn.tagExist = function (val) {
		return (jQuery.inArray(val, $(this)) >= 0); //true when tag exists, false when not
	};

	// clear all existing tags and import new ones from a string
	$.fn.importTags = function (str) {
		$('#' + id + '_tagsinput .tag').remove();
		$.fn.tagsInput.importTags(this, str);
	}

	$.fn.tagsInput = function (options) {
		var settings = jQuery.extend({
			interactive: true,
			defaultText: 'type a tag...',
			minChars: 0,
			width: '',
			autocomplete: { selectFirst: false },
			'hide': true,
			'delimiter': ',',
			'unique': true,
			removeWithBackspace: true,
			cssClass: '',
			linkFormat: '',
			autocompleteonly: false
		}, options);

		this.each(function () {
			if (settings.hide) {
				$(this).hide();
			}

			var id = $(this).attr('id')

			var data = jQuery.extend({
				pid: id,
				real_input: '#' + id,
				holder: '#' + id + '_tagsinput',
				input_wrapper: '#' + id + '_addTag',
				fake_input: '#' + id + '_tag'
			}, settings);


			delimiter[id] = data.delimiter;

			if (settings.onAddTag || settings.onRemoveTag || settings.onChange) {
				tags_callbacks[id] = new Array();
				tags_callbacks[id]['onAddTag'] = settings.onAddTag;
				tags_callbacks[id]['onRemoveTag'] = settings.onRemoveTag;
				tags_callbacks[id]['onChange'] = settings.onChange;
			}

			linkFormats[id] = settings.linkFormat;

			var markup = '<div id="' + id + '_tagsinput" class="tagsinput"><div id="' + id + '_addTag">';

			if (settings.interactive) {
				markup = markup + '<input maxlength="50" id="' + id + '_tag" value="" placeholder="' + settings.defaultText + '" />';
			}

			markup = markup + '</div><div class="tags_clear"></div></div>';

			$(markup).insertAfter(this);

			if (settings.width != "")
				$(data.holder).css('width', settings.width);

			if (settings.cssClass != "")
				$(data.holder).addClass(settings.cssClass);

			if ($(data.real_input).val() != '') {
				$.fn.tagsInput.importTags($(data.real_input), $(data.real_input).val());
			}
			if (settings.interactive) {
				$(data.holder).bind('click', data, function (event) {
					$(event.data.fake_input).focus();
				});

				if (settings.autocomplete_url != undefined) {
					autocomplete_options = { source: settings.autocomplete_url };
					for (attrname in settings.autocomplete) {
						autocomplete_options[attrname] = settings.autocomplete[attrname];
					}

					if (jQuery.autocomplete !== undefined) {
						$(data.fake_input).autocomplete(settings.autocomplete_url, { delay: 200, onItemSelect: function (li) {
							$(data.real_input).addTag(li.selectValue + "|" + li.extra[0], { focus: true, unique: (settings.unique) });
						}
						});
					}


				} else {
					// if a user tabs out of the field, create a new tag
					// this is only available if autocomplete is not used.
					$(data.fake_input).bind('blur', data, function (event) {
						if ($(event.data.fake_input).val() != '') {
							if ((event.data.minChars <= $(event.data.fake_input).val().length) && (!event.data.maxChars || (event.data.maxChars >= $(event.data.fake_input).val().length)))
								$(event.data.real_input).addTag($(event.data.fake_input).val() + "|0", { focus: true, unique: (settings.unique) });
						}
						return false;
					});

				}
				// if user presses enter or comma, create a new tag (unless disabled in settings)
				$(data.fake_input).bind('keypress', data, function (event) {
					if ((event.which == event.data.delimiter.charCodeAt(0) || event.which == 13) && !settings.autocompleteonly) {
						event.preventDefault();
						if ((event.data.minChars <= $(event.data.fake_input).val().length) && (!event.data.maxChars || (event.data.maxChars >= $(event.data.fake_input).val().length)))
							$(event.data.real_input).addTag($(event.data.fake_input).val() + "|0", { focus: true, unique: (settings.unique) });

						return false;
					}
				});
				//Delete last tag on backspace
				data.removeWithBackspace && $(data.fake_input).bind('keydown', function (event) {
					if (event.keyCode == 8 && $(this).val() == '') {
						event.preventDefault();
						var last_tag = $(this).closest('.tagsinput').find('.tag:last').text();
						var id = $(this).attr('id').replace(/_tag$/, '');
						last_tag = last_tag.replace(/[\s]+x$/, '');
						$('#' + id).removeTag(escape(last_tag));
						$(this).trigger('focus');
					}
				});
				$(data.fake_input).blur();
			} // if settings.interactive
			return false;
		});

		return this;

	};

	$.fn.tagsInput.updateTagsField = function (obj, tagslist) {
		var id = $(obj).attr('id');
		$(obj).val(tagslist.join(delimiter[id]));
	};

	$.fn.tagsInput.importTags = function (obj, val) {
		$(obj).val('');
		var id = $(obj).attr('id');
		var tags = val.split(delimiter[id]);
		for (i = 0; i < tags.length; i++) {
			$(obj).addTag(tags[i], { focus: false, callback: false });
		}
		if (tags_callbacks[id] && tags_callbacks[id]['onChange']) {
			var f = tags_callbacks[id]['onChange'];
			f(obj, tags[i]);
		}
	};

})(jQuery);
