
--creating tables
CREATE TABLE [dbo].[hdCategories] (
	[CategoryID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[Name] nvarchar (250) NOT NULL,
	[SectionID] int NULL,
	[InstanceID] int NOT NULL DEFAULT(0),
	[ForTechsOnly] bit NOT NULL DEFAULT (0),
	[ForSpecificUsers] bit NOT NULL DEFAULT (0),
	OrderByNumber int NOT NULL DEFAULT 1,
	FromAddress varchar(255) NULL,
	KBOnly bit NOT NULL DEFAULT 0,
	FromAddressInReplyTo bit NOT NULL DEFAULT 1,
	Notes varchar(max) NULL
)
GO

CREATE TABLE [dbo].[hdSections] (
	[SectionID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[Name] nvarchar (50) NOT NULL,
	[InstanceID] int NOT NULL DEFAULT(0),
	OrderByNumber int NOT NULL DEFAULT 1
)
GO

CREATE TABLE [dbo].[Instances] (
	[InstanceID] int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[Name] varchar (100) NOT NULL,
	[CustomerID] int NOT NULL,
	ValidTill datetime NOT NULL,
	CustomDomain varchar(100) NULL,
	AvangateSubscriptionID nvarchar(20) NULL,
	WidgetInstalled bit NOT NULL DEFAULT 0
)
GO

CREATE TABLE [dbo].[hdCategoryPermissions] (
	[UserID] [int] NOT NULL,
	[CategoryID] [int] NOT NULL 
)
GO

--now add complex primary key in [hdCategoryPermissions] table
ALTER TABLE [dbo].[hdCategoryPermissions] WITH NOCHECK ADD 
	CONSTRAINT [PK_hdCategoryPermissions] PRIMARY KEY CLUSTERED 
	(
		[UserID],
		[CategoryID]
	)
GO


CREATE TABLE [dbo].[hdCategorySubmitPermissions] (
	[UserID] [int] NOT NULL ,
	[CategoryID] [int] NOT NULL 
)
GO
--now add complex primary key in [hdCategoryPermissions] table
ALTER TABLE [dbo].[hdCategorySubmitPermissions] WITH NOCHECK ADD 
	CONSTRAINT [PK_hdCategorySubmitPermissions] PRIMARY KEY CLUSTERED 
	(
		[UserID],
		[CategoryID]
	)
GO

CREATE TABLE [dbo].[hdCategorySubmitPermissionsByCompany] (
	[CompanyID] [int] NOT NULL ,
	[CategoryID] [int] NOT NULL 
)
GO
--now add complex primary key in [hdCategoryPermissions] table
ALTER TABLE [dbo].[hdCategorySubmitPermissionsByCompany] WITH NOCHECK ADD 
	CONSTRAINT [PK_hdCategorySubmitPermissionsByCompany] PRIMARY KEY CLUSTERED 
	(
		[CompanyID],
		[CategoryID]
	)
GO

CREATE TABLE [dbo].[hdComments] (
	[CommentID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[IssueID] [int] NOT NULL ,
	[CommentDate] [datetime] NOT NULL DEFAULT (getdate()),
	[UserID] [int] NULL ,
	[Body] nvarchar(max) NOT NULL ,
	[ForTechsOnly] bit NOT NULL DEFAULT (0),
	[IsSystem] bit NOT NULL DEFAULT(0),
	Recipients nvarchar(3000) not null DEFAULT(''),
	EmailHeaders varchar(2000) NULL
)
GO

CREATE TABLE [dbo].[hdCommentTemplates] (
	[TemplateID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[Name] nvarchar(255) NULL,
	[Body] nvarchar(max) NOT NULL ,
)
GO

CREATE TABLE [dbo].[hdIssues] (
	[IssueID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[IssueDate] [datetime] NOT NULL DEFAULT (getdate()),
	[UserID] [int] NOT NULL ,
	[CategoryID] [int] NOT NULL ,
	[StartDate] datetime null,
	[ResolvedDate] [datetime] NULL ,
	[AssignedToUserID] [int] NULL ,
	[Subject] [nvarchar] (1000) NOT NULL ,
	[Body] nvarchar(max) NOT NULL ,
	[UpdatedByUser] [bit] NOT NULL DEFAULT (0),
	[UpdatedByPerformer] [bit] NOT NULL DEFAULT (0),
	[UpdatedForTechView] [bit] NOT NULL DEFAULT (0),
	[StatusID] [int] NOT NULL DEFAULT (1),
	[Priority] smallint NOT NULL DEFAULT (0),
	[TimeSpentInSeconds] int NOT NULL DEFAULT(0),
	[LastUpdated] datetime NOT NULL DEFAULT(getdate()),
	DueDate datetime NULL,
	Origin smallint NOT NULL default(0)
)
GO

CREATE TABLE [dbo].[KBArticles] (
	[ArticleID] [int] NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[CategoryID] [int] NOT NULL ,
	[Subject] [nvarchar] (1000) NOT NULL ,
	[Body] nvarchar(max) NOT NULL,
	[ForTechsOnly] [bit] NOT NULL DEFAULT (0),
	[OrderByNumber] int NOT NULL DEFAULT (1),
	DateCreated datetime NULL DEFAULT (getdate()),
	LastUpdated datetime NULL DEFAULT (getdate())
)
GO

CREATE TABLE [dbo].[hdOnBehalfIssues] (
	[IssueID] [int] NOT NULL PRIMARY KEY,
	[SubmittedByUserID] int NOT NULL
)
GO

CREATE TABLE hdMergedIssues (
	[IssueID] int NOT NULL,
	[IssueMergedID] int NOT NULL
)
ALTER TABLE hdMergedIssues WITH NOCHECK ADD CONSTRAINT [PK_hdMergedIssues] PRIMARY KEY CLUSTERED (
	IssueID,
	IssueMergedID
)
GO

CREATE TABLE [dbo].[hdCustomFields] (
	[FieldID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] [int] NOT NULL,
	[Type] [int] NOT NULL,
	[FieldName] nvarchar(100) NOT NULL,
	[ForTechsOnly] bit NOT NULL DEFAULT (0),
	[Mandatory] bit NOT NULL DEFAULT (0),
	[ShowInGrid] bit NOT NULL DEFAULT (1),
	[OrderByNumber] int NOT NULL DEFAULT (100),
	UsageType int NOT NULL DEFAULT 1
)
GO

CREATE TABLE hdIssueSchedule (
	IssueID int NOT NULL PRIMARY KEY,
	FreqType int NOT NULL,
	FirstRecurrenceDate datetime NOT NULL,
	PrevRepeatsCount int NOT NULL DEFAULT(0),
	ReopenOriginal bit not null default 0
)
GO

CREATE TABLE hdScheduledIssues (
	ScheduledIssueID int NOT NULL PRIMARY KEY,
	OriginalIssueID int NOT NULL
)
GO

CREATE TABLE [dbo].[hdCustomFieldValues] (
	[IssueID] [int] NOT NULL,
	[FieldID] [int] NOT NULL,
	[Value] sql_variant NOT NULL
)
GO
ALTER TABLE [dbo].[hdCustomFieldValues] WITH NOCHECK ADD 
	CONSTRAINT [PK_hdCustomFieldValues]PRIMARY KEY CLUSTERED 
	(
		[IssueID],
		[FieldID]
	)
GO

CREATE TABLE [dbo].[hdCustomFieldValuesForAssets] (
	[ItemID] [int] NOT NULL,
	[FieldID] [int] NOT NULL,
	[Value] sql_variant NOT NULL
)
GO
ALTER TABLE [dbo].[hdCustomFieldValuesForAssets] WITH NOCHECK ADD 
	CONSTRAINT [PK_hdCustomFieldValuesForAssets] PRIMARY KEY CLUSTERED 
	(
		[ItemID],
		[FieldID]
	)
GO

CREATE TABLE [dbo].[hdTags] (
	TagID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[Name] nvarchar(200) NOT NULL,
	InstanceID int NOT NULL
)
GO

CREATE TABLE [dbo].[hdIssueTags] (
	TagID int NOT NULL,
	IssueID int NOT NULL
)
ALTER TABLE hdIssueTags WITH NOCHECK ADD CONSTRAINT [PK_hdIssueTags] PRIMARY KEY CLUSTERED (TagID,IssueID)
GO

CREATE TABLE [dbo].[KBArticleTags] (
	TagID int NOT NULL,
	ArticleID int NOT NULL
)
ALTER TABLE KBArticleTags WITH NOCHECK ADD CONSTRAINT [PK_KBArticleTags] PRIMARY KEY CLUSTERED (TagID,ArticleID)
GO


CREATE TABLE [dbo].[hdCustomFieldOptions] (
	[OptionID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[FieldID] [int] NOT NULL,
	[OptionValue] nvarchar(500) NOT NULL,
	IsDefault bit not null default 0,
	OrderByNumber int NOT NULL DEFAULT 1,
	IsDisabled bit not null default 0
)
GO

CREATE TABLE [dbo].[CustomFieldCategories] (
	[FieldID] [int] NOT NULL,
	[CategoryID] int NOT NULL
)
ALTER TABLE CustomFieldCategories WITH NOCHECK ADD CONSTRAINT [PK_CustomFieldCategories] PRIMARY KEY CLUSTERED (
	FieldID,
	CategoryID
)

CREATE TABLE [dbo].[hdNews] (
	[NewsID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[NewsDate] [datetime] NOT NULL DEFAULT (getdate()),
	[UserID] [int] NOT NULL ,
	[Subject] [nvarchar] (100) NOT NULL ,
	[Body] nvarchar(max) NOT NULL
)
GO

CREATE TABLE [dbo].[hdSettings] (
	[InstanceID] int NOT NULL DEFAULT(0) PRIMARY KEY,
	[EmailNotificationsEnabled] [bit] NOT NULL DEFAULT (0),
	[SMTPServer] varchar (50) NULL ,
	[SMTPAuthRequired] [bit] NOT NULL  DEFAULT (0),
	[SMTPUserName] varchar (50) NULL ,
	[SMTPPassword] varchar (50) NULL ,
	[SMTPUseSSL] bit NOT NULL  DEFAULT (0) ,
	[FromAddress] varchar (150) NULL ,
	ReplyToAddress varchar (50) NULL,
	FromName varchar(50) NULL,
	UseFromNameForUserOriginatedMsgs bit not null DEFAULT 0,
	AllowUserRegistration bit NOT NULL  DEFAULT (1),
	SMTPPort int NOT NULL DEFAULT (25),
	MailCheckerNewIssuesCategoryID int NOT NULL DEFAULT (1),
	MailCheckerAllowUnregisteredUsers bit NOT NULL DEFAULT (1),
	RestrictIssueDeletion bit NOT NULL DEFAULT (0),
	Lang varchar (10) NOT NULL DEFAULT ('en-US'),
	MaxAttachSize int NOT NULL DEFAULT (1000),
	RestrictIssueClosing bit NOT NULL DEFAULT (0),
	NewIssuesDefaultCategoryID int NOT NULL DEFAULT 1,
	AllowGuestSubmission bit default 1,
	AllowAnonymousKBAccess bit default 1,
	SeeEveryonesIssues bit default 0,
	UseDefaultSmtp bit default 0,
	HeaderBGColor char(6) NOT NULL DEFAULT('34383E'),
	MenuBarBGColor char(6) NOT NULL DEFAULT('4C5B6B'),
	HeaderTextColor char(6) NOT NULL DEFAULT('FFFFFF'),
	LogoImage [image] NULL,
	SendTechniciansEmailOnNewIssue bit NOT NULL DEFAULT(1),
	DisableKB bit NOT NULL DEFAULT(0),
	AutoLoginSharedSecret varchar(20) NULL,
	MenuBarTextColor char(6) NOT NULL DEFAULT('C7D4E2'),
	DisableCloseNotification bit NOT NULL DEFAULT 1,
	Title nvarchar(500) NOT NULL DEFAULT 'helpdesk',
	DisableAssets bit not null default 0,
	DaysToCloseAfter int NOT NULL DEFAULT(0),
	CRMRootUrl varchar(1000) not null DEFAULT '',
	CRMSharedKey varchar(255) not null DEFAULT '',
	SameCompanyUserPermission bit not null default(0),
	HidePoweredBy bit not null default(0),
	DisableAvatars bit NOT NULL DEFAULT 0,
	SendAdminEmailOnNewIssue bit NOT NULL DEFAULT 1,
	SendTechSMSOnNewIssue bit NOT NULL DEFAULT 0,
	AutoAssignFirstReplier bit not null default 1,
	DontReopenOnReply bit NOT NULL default 0,
	NotifyAllTechsOnReply bit NOT NULL default 0,
	NewTicketNote nvarchar(max) NULL,
	CustomCSS varchar(max) NULL,
	DisableAutoTimer bit not null default 0,
	DontAddCCToSubscribers bit not null default 0,
	WorkHourFrom int null,
	WorkHourTo int null,
	EmailOtherTechsOnTakeover bit not null default 0,
	EnableSaml bit not null default 0,
	SamlEndpoint varchar(200),
	SamlCertificate varchar(2000),
	DisableTicketConfirmationNotification bit NOT NULL DEFAULT 0,
	TimezoneID varchar(50) NOT NULL default('Eastern Standard Time'),
	ActiveTabTextColor char(6) NOT NULL DEFAULT('FFFFFF'),
	RemoteLoginURL varchar(500) NULL,
	NoAttachmentsInNotifications bit NOT NULL DEFAULT 0,
	DisableAutologinLinks bit NOT NULL DEFAULT 0,
	ExtractOriginalSenderFromFwd bit NOT NULL DEFAULT 1,
	FaviconImage [image] NULL,
	EnableGoogleLogin bit NOT NULL DEFAULT 0,
	DisableUsernameEmailEdit bit NOT NULL DEFAULT 0,
	KBTitle varchar(max) NULL,
	KBCss varchar(max) NULL,
	KBHeadCode varchar(max) NULL,
	SendTechEmailOnKBUpdate bit not null default 0,
	HideLoginButtonWhenSAML bit not null default 0,
	CustomJS varchar(max) NULL,
	EnableMultiAsssign bit not null default 0,
	LogoURL varchar(100) NULL
)
GO

CREATE TABLE [dbo].[SecondaryAssignees] (
	[IssueID] int NOT NULL,
	[UserID] int NOT NULL
)
GO
--now add complex primary key in [hdIssueSubscribers] table
ALTER TABLE [dbo].[SecondaryAssignees] WITH NOCHECK ADD 
	CONSTRAINT [PK_SecondaryAssignees] PRIMARY KEY CLUSTERED ([IssueID],[UserID])
GO

CREATE TABLE [dbo].[ReportSchedule] (
	[InstanceID] int NOT NULL DEFAULT(0) PRIMARY KEY,
	ReportModelXml varchar(max) not null,
	LastSent datetime null
)

CREATE TABLE [dbo].[EmailTemplates] (
	[InstanceID] int NOT NULL DEFAULT(0) PRIMARY KEY,
	EmailTemplate nvarchar (3000) NOT NULL DEFAULT '',
	EmailSubjectTemplate nvarchar (500) NOT NULL DEFAULT '',
	NewIssueEmailSubject nvarchar(500) NOT NULL DEFAULT '',
	NewIssueEmailTemplate nvarchar(3000) NOT NULL DEFAULT '',
	WelcomeEmailSubj nvarchar (500) NOT NULL DEFAULT '',
	WelcomeEmailBody nvarchar (3000) NOT NULL DEFAULT '',
	NewIssueConfirmationSubj nvarchar(500) NOT NULL DEFAULT '',
	NewIssueConfirmationBody nvarchar(3000) NOT NULL DEFAULT '',
	ClosedTicketEmailTemplate nvarchar(3000) NOT NULL DEFAULT '',
	ClosedTicketEmailSubject nvarchar(500) NOT NULL DEFAULT ''
)
GO

CREATE TABLE hdPOPServers (
	ServerID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	HostName varchar (200) NULL,
	Port int NOT NULL DEFAULT (110),
	POPLogin varchar (100) NULL,
	POPPassword varchar (100) NULL,
	UseSSL bit NOT NULL DEFAULT (0),
	NewTicketsCategoryID int NULL,
	IsImap bit NOT NULL DEFAULT 0,
	Disabled bit NOT NULL DEFAULT 0,
	ImapMarkSeenNotDelete bit NOT NULL DEFAULT 0
)

CREATE TABLE [dbo].[hdStatus] (
	[StatusID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	InstanceID int NULL,
	[Name] nvarchar (50) NOT NULL,
	[ButtonCaption] nvarchar(50) NULL,
	[ForTechsOnly] bit NOT NULL DEFAULT (1),
	[StopTimeSpent] bit not null DEFAULT (0)
)
GO

CREATE TABLE StatusCategories (
	StatusID int NOT NULL,
	[CategoryID] int NOT NULL
)
ALTER TABLE StatusCategories WITH NOCHECK ADD CONSTRAINT [PK_StatusCategories] PRIMARY KEY CLUSTERED (
	StatusID,
	CategoryID
)

CREATE TABLE [dbo].[hdUsers] (
	[UserID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[UserName] varchar (200) NOT NULL ,
	[Email] varchar (200) NOT NULL ,
	[Password] varchar (50) NOT NULL ,
	[IsAdministrator] [bit] NOT NULL DEFAULT (0),
	[FirstName] nvarchar(255) NOT NULL DEFAULT (''),
	[LastName] nvarchar(255) NOT NULL DEFAULT (''),
	CompanyID int NULL,
	[Notes] nvarchar(4000) NULL,
	[Phone] varchar(50) NULL,
	[Location] nvarchar(1000) NULL,
	[SendEmail] bit not null default 1,
	Disabled bit not null default 0,
	LastSeen datetime null,
	IPAddress varchar(50) null,
	HostName varchar(255) null,
	Lang varchar(10) NOT NULL DEFAULT(''),
	UserAgent varchar(255) null,
	[Signature] nvarchar(2000) null,
	Greeting nvarchar(500) null,
	DepartmentID int null,
	IsManager bit not null DEFAULT(0),
	TechTutorialShown bit not null default(0),
	AdminTutorialShown bit not null default(0),
	PushToken varchar(255) NULL,
	TwoFactorAuthEnabled bit NOT NULL DEFAULT (0),
	PushPlatform varchar(10) NULL ,
	TimezoneID varchar(50) NULL,
	LastPasswordChange datetime null
)
GO
--now add constraint - unique username/instance - in hdUsers table
ALTER TABLE [dbo].[hdUsers] WITH NOCHECK ADD 
	CONSTRAINT [IX_hdUsers_UniqueNameInstance] UNIQUE  NONCLUSTERED 
	(
		[username],
		InstanceID
	)
GO

CREATE TABLE [dbo].[UserAvatars] (
	[UserID] [int] NOT NULL PRIMARY KEY,
	[ImageData] [image] NULL
)

CREATE TABLE [dbo].[hdCompanies] (
	CompanyID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[Name] nvarchar(255) NOT NULL,
	CRMID int NULL,
	[EmailDomain] nvarchar(255) NULL,
	Notes nvarchar(2000) NULL
)

CREATE TABLE [dbo].[Departments] (
	DepartmentID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[Name] nvarchar(255) NOT NULL
)

CREATE TABLE [dbo].[hdIssueSubscribers] (
	[IssueID] [int] NOT NULL,
	[UserID] [int] NOT NULL
)
GO
--now add complex primary key in [hdIssueSubscribers] table
ALTER TABLE [dbo].[hdIssueSubscribers] WITH NOCHECK ADD 
	CONSTRAINT [PK_hdIssueSubscribers] PRIMARY KEY CLUSTERED 
	(
		[IssueID],
		[UserID]
	)
GO

CREATE TABLE [dbo].[hdFileAttachments](
		[FileID] [int] IDENTITY(1,1) NOT NULL PRIMARY KEY,
		[IssueID] [int] NOT NULL,
		[CommentID] int NOT NULL DEFAULT (0),
		[UserID] [int] NOT NULL,
		[FileName] nvarchar(255) NOT NULL,
		[FileData] [image] NULL,
		GoogleDriveUrl varchar(255) NULL,
		DropboxUrl varchar(500) NULL,
		FileHash binary(16) NULL,
		FileSize int NOT NULL DEFAULT 0,
		HiddenFromKB bit NOT NULL DEFAULT 0,
		HiddenFromTickets bit NOT NULL DEFAULT 0,
		AssetID int NULL
)

CREATE TABLE DuplicateFiles (
	FileID int NOT NULL,
	CommentID int NOT NULL
)
ALTER TABLE DuplicateFiles WITH NOCHECK ADD
	CONSTRAINT [PK_DuplicateFiles] PRIMARY KEY CLUSTERED (FileID, CommentID)
GO


--ticket-to-ticket links
CREATE TABLE hdLinkedIssues (
	Issue1ID int NOT NULL,
	Issue2ID int NOT NULL
)
ALTER TABLE hdLinkedIssues WITH NOCHECK ADD 
	CONSTRAINT [PK_hdLinkedIssues] PRIMARY KEY CLUSTERED ([Issue1ID],[Issue2ID])
GO


--asset management tables
CREATE TABLE [dbo].[hdAssets] (
	ItemID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	InstanceID int NOT NULL,
	ModelName nvarchar(1000) NOT NULL,
	TypeID int NOT NULL,
	ManufacturerID int NOT NULL,
	Quantity int NOT NULL DEFAULT(0),
	SupplierID int NOT NULL,
	SerialNumber nvarchar(200) NULL,
	Location nvarchar(500) NULL,
	Comments nvarchar(1000) NULL
)
CREATE TABLE [dbo].[hdAssetManufacturers] (
	ManufacturerID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	InstanceID int NOT NULL,
	[Name] nvarchar(1000) NOT NULL
)
CREATE TABLE [dbo].[hdAssetSuppliers] (
	SupplierID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	InstanceID int NOT NULL,
	[Name] nvarchar(1000) NOT NULL
)
CREATE TABLE [dbo].[hdAssetTypes] (
	TypeID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	InstanceID int NOT NULL,
	[Name] nvarchar(1000) NOT NULL
)
CREATE TABLE [dbo].[hdAssetUsers] (
	ItemID int NOT NULL,
	UserID int NOT NULL
)
ALTER TABLE [dbo].[hdAssetUsers] WITH NOCHECK ADD 
	CONSTRAINT [PK_hdAssetUsers] PRIMARY KEY CLUSTERED ([ItemID],[UserID])
GO


CREATE TABLE dbo.hdAssetsIssues	(
	IssueID int NOT NULL,
	AssetID int NOT NULL
) 
GO
ALTER TABLE [dbo].hdAssetsIssues WITH NOCHECK ADD 
	CONSTRAINT [PK_hdAssetsIssues] PRIMARY KEY CLUSTERED ([IssueID], [AssetID])
GO


CREATE TABLE [dbo].[Integrations] (
	[InstanceID] int NOT NULL PRIMARY KEY,
	BitbucketEnabled bit NOT NULL DEFAULT 0,
	BitbucketUsername varchar(50) null,
	BitbucketPassword varchar(50) null,
	HipChatEnabled bit NOT NULL DEFAULT 0,
	HipChatAuthToken varchar(50) null,
	HipChatRoomId int null,
	JiraEnabled bit NOT NULL DEFAULT 0,
	JiraUsername varchar(50) null,
	JiraPassword varchar(50) null,
	JiraUrl varchar(100) null,
	DropboxEnabled bit NOT NULL DEFAULT 0,
	DropboxAppID varchar(25) NULL,
	SlackEnabled bit NOT NULL DEFAULT 0,
	SlackWebhookURL varchar(100) null,
	GitHubEnabled bit not null default 0,
	GitHubUsername varchar(50) null,
	GitHubPassword varchar(50) null,
	GithubEnterpriseUrl varchar(100) null,
	UserLookupEnabled bit NOT NULL DEFAULT 0,
	UserLookupURL varchar(500) NULL,
	UserLookupBasicAuthEnabled bit NOT NULL DEFAULT 0,
	UserLookupUsername varchar(50) NULL,
	UserLookupPassword varchar(50) NULL,
	AsanaEnabled bit not null default 0,
	AsanaAccessToken varchar(300) null,
	TrelloEnabled bit not null default 0,
	VSOEnabled bit not null default 0,
	VSOAccountName varchar(50) null,
	VSOUsername varchar(50) null,
	VSOToken varchar(100) null,
	HarvestEnabled bit not null default 0,
	HarvestUsername varchar(50) null,
	HarvestPassword varchar(50) null,
	HarvestAccount varchar(50) null,
	ZapierEnabled bit not null default 0,
	GoogleDriveEnabled bit NOT NULL DEFAULT 0,
	GoogleAPIKey varchar(100) null,
	GoogleOAuthClientID varchar(100) null
)
GO

CREATE TABLE [dbo].[IntegrationTicketData] (
	IssueID int NOT NULL PRIMARY KEY,
	BitbucketIssueUrl varchar(300) null,
	JiraIssueUrl varchar(300) null,
	GitHubIssueUrl varchar(300) null,
	AsanaTaskUrl varchar(300) null,
	TrelloCardUrl varchar(300) null,
	VSOWorkItemUrl varchar(300) null,
	HarvestProjectUrl varchar(300) null
)
GO

CREATE TABLE AutomationRules (
	RuleID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	InstanceID int NOT NULL,
	RuleXml nvarchar(max),
	OrderByNumber int NOT NULL DEFAULT 1
)
GO



CREATE TABLE OverdueAutomationLog (
	IssueID int NOT NULL,
	RuleID int NOT NULL
)
GO
ALTER TABLE OverdueAutomationLog WITH NOCHECK ADD 
	CONSTRAINT [PK_OverdueAutomationLog] PRIMARY KEY CLUSTERED (IssueID, RuleID)
GO



CREATE TABLE NonupdatedAutomationLog (
	IssueID int NOT NULL,
	RuleID int NOT NULL
)
GO
ALTER TABLE NonupdatedAutomationLog WITH NOCHECK ADD 
	CONSTRAINT [PK_NonupdatedAutomationLog] PRIMARY KEY CLUSTERED (IssueID, RuleID)
GO

CREATE TABLE [dbo].[DeletionLog]
(
	LogID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
    [InstanceID] INT NOT NULL, 
    [UserID] INT NOT NULL, 
    [Description] NVARCHAR(500) NOT NULL,
	[Date] [datetime] NOT NULL DEFAULT (getdate())
)

CREATE TABLE [dbo].[ZapierSubscriptions]
(
	[SubscriptionID] INT IDENTITY (1, 1) NOT NULL PRIMARY KEY, 
    [InstanceID] INT NOT NULL, 
    [EventID] INT NOT NULL, 
    [TargetURL] VARCHAR(500) NOT NULL
)


--create default STATUSes DO NOT REMOVE!!!!!!!!!!!!!!!!
INSERT INTO hdStatus (Name) VALUES ('New')
go

INSERT INTO hdStatus (Name) VALUES ('In progress')
go

INSERT INTO hdStatus (Name) VALUES ('Resolved')
go

--cut-- 
--create TWO PRE-DEFINED USERS
INSERT INTO hdUsers (username, email, password, IsAdministrator)
VALUES ('admin', 'admin@testmail', '21232f297a57a5a743894a0e4a801fc3', 1)
GO

INSERT INTO hdUsers (username, email, password, IsAdministrator)
VALUES ('client', 'client@testmail', '62608e08adc29a8d6dbc9754e659f125', 0)
GO

--create pre-defined sections
INSERT INTO hdSections (Name) VALUES ('Sales')
go

INSERT INTO hdSections (Name) VALUES ('Technical')
go

--create PRE-DEFINED CATEGORIES
INSERT INTO hdCategories (Name, SectionID) VALUES ('Bug reports', 2)
go

INSERT INTO hdCategories (Name, SectionID) VALUES ('Feature requests', 2)
go

INSERT INTO hdCategories (Name, SectionID) VALUES ('Hardware issues', 2)
go

INSERT INTO hdCategories (Name, SectionID) VALUES ('Other', 2)
go

INSERT INTO hdCategories (Name, SectionID) VALUES ('Payment issues', 1)
go

INSERT INTO hdCategories (Name, SectionID) VALUES ('Pre-sales questions', 1)
go

--creating foreign keys
ALTER TABLE [dbo].[hdCategoryPermissions] ADD 
	CONSTRAINT [FK_hdCategoryPermissions_hdCategories] FOREIGN KEY([CategoryID]) REFERENCES [dbo].[hdCategories] ([CategoryID]),
	CONSTRAINT [FK_hdCategoryPermissions_hdUsers] FOREIGN KEY([UserID]) REFERENCES [dbo].[hdUsers] ([UserID])
GO

ALTER TABLE [dbo].[hdComments] ADD 
	CONSTRAINT [FK_hdComments_hdIssues] FOREIGN KEY([IssueID]) REFERENCES [dbo].[hdIssues] ([IssueID]),
	CONSTRAINT [FK_hdComments_hdUsers] FOREIGN KEY([UserID]) REFERENCES [dbo].[hdUsers] ([UserID])
GO

ALTER TABLE [dbo].[hdIssues] ADD 
	CONSTRAINT [FK_hdIssues_hdCategories] FOREIGN KEY([CategoryID]) REFERENCES [dbo].[hdCategories] ([CategoryID]),
	CONSTRAINT [FK_hdIssues_hdStatus] FOREIGN KEY([StatusID]) REFERENCES [dbo].[hdStatus] ([StatusID]),
	CONSTRAINT [FK_hdIssues_hdUsers1] FOREIGN KEY([UserID]) REFERENCES [dbo].[hdUsers] ([UserID]),
	CONSTRAINT [FK_hdIssues_hdUsers3] FOREIGN KEY([AssignedToUserID]) REFERENCES [dbo].[hdUsers] ([UserID])
GO

ALTER TABLE [dbo].[hdIssueSubscribers] ADD
	CONSTRAINT [FK_hdIssueSubscribers_hdIssues] FOREIGN KEY([IssueID]) REFERENCES [dbo].[hdIssues] ([IssueID]),
	CONSTRAINT [FK_hdIssueSubscribers_hdUsers] FOREIGN KEY([UserID]) REFERENCES [dbo].[hdUsers] ([UserID])
GO

--indexes
CREATE INDEX IX_hdComments_IssueID ON hdComments (IssueID)
CREATE INDEX IX_hdComments_UserID ON hdComments (UserID)
CREATE INDEX IX_hdIssues_CategoryID ON hdIssues (CategoryID)
CREATE INDEX IX_hdIssues_UserID ON hdIssues (UserID)
CREATE INDEX IX_hdIssues_StatusID ON hdIssues (StatusID)
CREATE INDEX IX_hdFileAttachments_IssueID ON hdFileAttachments (IssueID)
CREATE INDEX IX_hdIssues_AssignedToUserID ON hdIssues (AssignedToUserID)
CREATE INDEX IX_KBArticles_CategoryID ON KBArticles (CategoryID)

--create fulltextindexes
-- if fulltext components are installed
declare @installed sql_variant
select @installed=SERVERPROPERTY('IsFullTextInstalled')
IF(@installed = 1)
begin

-- creating default catalog
CREATE FULLTEXT CATALOG hdFulltextCatalog AS DEFAULT

-- creating hdComments index
declare @commentsConstraint sysname
declare @commentIndexCmd varchar(1000)

SELECT @commentsConstraint=CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME='hdComments' AND CONSTRAINT_TYPE='PRIMARY KEY'

SET @commentIndexCmd = 'CREATE FULLTEXT INDEX ON hdComments ([Body])
						KEY INDEX ' + @commentsConstraint						
EXEC(@commentIndexCmd)


-- creating hdIssues index
declare @issuesConstraint sysname
declare @issueIndexCmd varchar(1000)

SELECT @issuesConstraint=CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME='hdIssues' AND CONSTRAINT_TYPE='PRIMARY KEY'

SET @issueIndexCmd = 'CREATE FULLTEXT INDEX ON hdIssues ([Subject], [Body])
						KEY INDEX ' + @issuesConstraint
						
EXEC(@issueIndexCmd)
end